package ssc;

/**
 * The base class for all automats
 * @author Medvinskiy M.
 */
public abstract class Automat{

    /**
    * State
    */
    private int y0;

    /**
    * Automat's states titles
    */
    private String[] states;

    /**
    * Nuber of automat's possible states
    */
    private int statesCount;
    
    /**
    * Sets the count of automat's possible states
    * @param count of automat possible states
    */
    public void setStatesCount(int count) {
        statesCount = count;
        states = new String[statesCount];
    };
    
    /**
    * Returns the name of specified state
    */
    public String getStateName(int index) {
        if (!(index >= statesCount)) return states[index];
        return "";
    };

    /**
    * Sets the name of specified state
    */
    public void setStateName(int index, String name) {
        states[index] = name;
    };
    
    /**
    * Returns the current state
    */
    protected int getState() {return y0;};

    /**
    * Sets the current state
    */
    protected void setState(int y) {y0 = y;};

    /**
    * Automat's name
    */
    private String name;
    
    /**
    * Returns automat's name
    */
    protected String getName() {return name;};
    
    /**
    * Sets the automat's name
    * @param n name
    */
    protected void setName(String n) {name = n;};

    /**
    * Automat's type (A0, A1, A2, etc )
    */
    private String automatType;
    
    /**
    * Returns automat's type
    */
    protected String getType() {return automatType;};

    /**
    * Sets the type of automat
    * @param t automat's type
    */
    protected void setType(String t) {automatType = t;};

    /**
    * Indicates if the actions of this automat are logged
    */
    private boolean isLogged;
    
    /**
    * Returns if this automat is logged
    */
    protected boolean getLogged() {return isLogged;};

    /**
    * Sets the "isLogged" parametr
    */
    protected void setLogged(boolean b) {isLogged = b;};
    
    /**
    *   .
    *    ,    
    *   <code>to</code>   
    * <code>from</code>,   <code>e</code>
    * @param to  
    * @param from  
    * @param e  
    */
    protected void logTrans(int to, int from, int e) {
        if (!isLogged) return;
        
        java.util.Date time = new java.util.Date();
        time = java.util.Calendar.getInstance(java.util.Locale.getDefault()).getTime();
        Log.writeln(time.toString() + "| T " + automatType + 
          " <" + name + ">: has changed state from " + from + " (\""+ getStateName(from) +
          "\") to " + to + " (\"" + getStateName(to) + "\") | event: " + e);
    };

    /**
    *    .
    *    ,     
    * zi()
    * @param z   
    * @param comment  
    */
    protected void logZ(int z, String comment) {
        if (!isLogged) return;

        java.util.Date time = new java.util.Date();
        time = java.util.Calendar.getInstance(java.util.Locale.getDefault()).getTime();
        Log.writeln(time.toString() + "| Z " + automatType + 
          " <" + name + ">: z" + z + " | " + comment);
    };

    /**
    *   .
    *    ,      state
    * @param state 
    */
    protected void logLaunch(int state) {
        if (!isLogged) return;

        java.util.Date time = new java.util.Date();
        time = java.util.Calendar.getInstance(java.util.Locale.getDefault()).getTime();
        Log.writeln(time.toString() + "| L " + automatType + 
          " <" + name + ">: launched in state " + state + " (\"" + getStateName(state) + "\")");
    };

    /**
    *  .   
    *  ,   <code>e</code>, 
    *    
    * @param e  
    */
    public abstract void A(int e);
};
