package ssc;

import java.awt.*;
import java.util.*;

/**
 * This class implements the logic of "burst" which is a set of flashes
 */
public class Burst {

    /**
     * This class implements the logic of flash which is bursting fragment of unit
     */
    private class Flash {
        /**
         * This vector determines the direction of the flashe's movement
         */
        public Vector2 vec;

        /**
         * The owner of the flash
         */
        public Burst owner;

        /**
         * Flashe color
         */
        public Color color;

        /**
         * Construct flash of specified {@link Burst}
         * @param owner flashe's owner
         */
        public Flash(Burst owner) {
            this.owner = owner;
            vec = new Vector2();
            double l = 1 + Math.random();
            double angle = Math.random() * 2 * Math.PI;
            vec.x = 2 * l * Math.sin(angle);
            vec.y = l * Math.cos(angle);
            vec.z = 0;

            double k = Math.random();
            if (k < 0.5) color = Color.red;
            if (k >= 0.5) color = Color.yellow;
        };
    };

    /**
     * Bursting unit
     */
    private GameObject burstUnit;
    private double burstDeg;

    private boolean alive;

    /**
     * Returns if the burst is still continuing
     */
    public boolean isAlive() {
        return alive;
    }

    /**
     * Vector of flashes
     */
    private Vector flashes = new Vector();

    /**
     * Construct new burst of specified {@link GameObject}
     * @param object bursting {@link GameObject}
     */
    public Burst(GameObject object) {
        burstUnit = object;
        burstDeg = 0;
        alive = true;
        for (int i = 0; i < 40; i++) {
            flashes.add(new Flash(this));
        }
    };
    
    /**
     * Paint a burst
     * @param g Graphic context
     */
    public void paint(Graphics g) {
        Color old = g.getColor();
        g.setColor(Color.yellow);

        burstDeg = Math.sqrt(burstDeg * burstDeg + 20);
        if (burstDeg > 40) alive = false;
        int dx, dy;
        int x = (int) burstUnit.getX();
        int y = (int) burstUnit.getY();
        int r = (int) (30 / (burstDeg + 1));
        for (int i = 0; i < flashes.size(); i++) {
            Flash flash = (Flash) flashes.elementAt(i);
            dx = (int) (burstDeg * flash.vec.x);
            dy = (int) (burstDeg * flash.vec.y);
            g.setColor(flash.color);
            g.fillOval(dx + x - r, dy + y - r, 2 * r, 2 * r);
        }

        g.setColor(old);
    };
}
