package ssc;

import java.awt.*;

/**
 * <code>GameObject</code>
 * This class is a base class for all objects in the game: buildings and units
 */
public abstract class GameObject {

    /**
     * ,   ,   
     */
    private Rectangle ovalRect = new Rectangle();

    protected void setOvalRect(double x, double y, double width, double height) {
        ovalRect.x = (int) x;
        ovalRect.y = (int) y;
        ovalRect.width = (int) width;
        ovalRect.height = (int) height;
    };


    /**
     * ,       ,   
     */
    private Rectangle healthRect = new Rectangle();

    protected void setHealthRect(double x, double y, double width, double height) {
        healthRect.x = (int) x;
        healthRect.y = (int) y;
        healthRect.width = (int) width;
        healthRect.height = (int) height;
    };

    /**
     *  
     */
    private Point center = new Point();

    /**
     *   
     */
    protected void setCenter(double x, double y) {
        center.x = (int) x;
        center.y = (int) y;
    }

    /**
     * The name of GameObject
     */
    private String name;

    /**
     *   
     */
    protected String getName() {
        return name;
    };

    /**
     *   
     */
    protected void setName(String n) {
        name = n;
    };

    /**
     *  ,    
     */
    private BattleField field;

    protected BattleField getField() {
        return field;
    };
    protected void setField(BattleField field) {
        this.field = field;
    };

    /**
     *  ,    
     */
    private int team;

    /**
     *   
     */
    protected int getTeam() {
        return team;
    };

    /**
     *   
     */
    protected void setTeam(int t) {
        team = t;
    };

    /**
     *      x
     */
    private double x;

    /**
     *      y
     */
    private double y;


    protected double getX() {
        return x;
    };
    protected double getY() {
        return y;
    };

    protected void setX(double x) {
        this.x = x;
    };
    protected void setY(double y) {
        this.y = y;
    };

    /**
     *    
     */
    private int sightRange;

    protected int getSightRange() {
        return sightRange;
    };

    protected void setSightRange(int s) {
        sightRange = s;
    };

    /**
     *    
     */
    private int attackRange;

    /**
     *     
     */
    protected int getAttackRange() {
        return attackRange;
    };

    protected void setAttackRange(int r) {
        attackRange = r;
    };

    /**
     *  (   )
     */
    private int health;

    protected int getHealth() {
        return health;
    };

    protected void setHealth(int h) {
        health = h;
    };

    /**
     *     ?
     */
    private boolean controlled;

    protected boolean isControlled() {
        return controlled;
    }

    protected void setControlled(boolean c) {
        controlled = c;
    }

    protected void damage(int d) {
        health = health - d;
    };

    /**
     *     x;
     */
    protected int getIntX() {
        return (int) x;
    };

    /**
     *     y;
     */
    protected int getIntY() {
        return (int) y;
    };

    //  
    private double directX;

    private double directY;

    protected double getDirectX() {
        return directX;
    };
    protected double getDirectY() {
        return directY;
    };

    protected void setDirectX(double dx) {
        directX = dx;
    };
    protected void setDirectY(double dy) {
        directY = dy;
    };

    protected void setDirect(double x, double y) {
        directX = x;
        directY = y;
    };

    /**
     *  x   
     */
    private int destinationX;

    /**
     *  y   
     */
    private int destinationY;

    protected int getDestinationX() {
        return destinationX;
    };
    protected int getDestinationY() {
        return destinationY;
    };

    protected void setDestinationX(int d) {
        destinationX = d;
    };
    protected void setDestinationY(int d) {
        destinationY = d;
    };

    protected void setDestination(int x, int y) {
        destinationX = x;
        destinationY = y;
    };

    private boolean selected;

    protected boolean ifSelected() {
        return selected;
    };
    protected void select() {
        selected = true;
    };
    protected void deselect() {
        selected = false;
    };

    protected Image img;

    /**
     *    
     */
    private boolean alive;

    protected boolean isAlive() {
        return alive;
    };

    protected void setAlive(boolean a) {
        alive = a;
    };

    /**
     *   ( )
     */
    private int curMode;

    /**
     *   
     */
    protected int getCurMode() {
        return curMode;
    };

    /**
     *    
     */
    protected void setCurMode(int c) {
        curMode = c;
    };

    /**
     *       
     */
    public abstract void move();

    /**
     *      ,   .
     *    draw,       
     */
    public void paint(Graphics g) {
        Color old = g.getColor();
        Graphics2D g2 = (Graphics2D) g;
        if (selected && (this instanceof Unit)) {
            g2.setColor(Color.green);
            g2.drawOval((int) (x + ovalRect.x), (int) (y + ovalRect.y), (int) (ovalRect.width), (int) (ovalRect.height));
        }

        draw(g);

        if (selected && (this instanceof Unit)) {
            g2.setColor((health < 33) ? Color.red : ((health < 66) ? Color.yellow : Color.green));
            g2.fillRect((int) x + healthRect.x, (int) y + healthRect.y, healthRect.width * health / 100, healthRect.height);
            g2.setColor(Color.gray);
            g2.drawRect((int) x + healthRect.x, (int) y + healthRect.y, healthRect.width, healthRect.height);
        }
        g2.setColor(old);
    }

    /**
     *   (     )
     */
    public void draw(Graphics g) {
        g.drawImage(img, (int) x + center.x, (int) y + center.y, null);
        //drawGameObjectBounds(g);
    }

    /**
     * ,    (xx, yy) 
     */
    public abstract boolean ifin(int xx, int yy);

    /**
     * Draw GameObjects bounds
     * @param g
     */
    public void drawGameObjectBounds(Graphics g) {
        Color oldColor = g.getColor();
        g.setColor(Color.blue);
        g.drawRect((int) x + center.x - 1, (int) y + center.y - 1, img.getWidth(null) + 2, img.getHeight(null) + 2);
        g.drawRect((int) x - 1, (int) y - 1, 2, 2);
        g.setColor(oldColor);
    }
}

;

