package ssc;

/**
 * This class supports operations with 3D vector
 */
public class Vector2 {
    /**
     * x coordinate of the current vector
     */
    public double x;

    /**
     * y coordinateof the current vector
     */
    public double y;

    /**
     * z coordinateof the current vector
     */
    public double z;

    /**
     * Default constructor
     */
    public Vector2() {
        x = 0;
        y = 0;
        z = 0;
    }
    
    /**
     * Creates new vector with specified coordinates
     * @param x x-coordinate
     * @param y y-coordinate
     * @param z z-coordinate
     */
    public Vector2(double x, double y, double z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }
    
    /**
     * Calculates an angle between this vector and specified
     */
    public double Angle(Vector2 v2) {
        double v3x = x - v2.x;
        double v3y = y - v2.y;
        double v3z = z - v2.z;
        double side1 = Math.sqrt(x * x + y * y + z * z);
        double side2 = Math.sqrt(v2.x * v2.x + v2.y * v2.y + v2.z * v2.z);
        double side3q = (v3x * v3x + v3y * v3y + v3z * v3z);
        double d = (side3q - side1 * side1 - side2 * side2) / (2 * side1 * side2);
        return Math.acos(d);
    }
    
    /**
     * Returns determinant of ((A, B),(C, D)) matrix
     */
    public static double det2(double A, double B, double C, double D) {
        return A * D - B * C;
    }

    /**
     * Returns determinant of 3x3-matrix
     */
    public static double det3(double A1, double A2, double A3,
                              double B1, double B2, double B3,
                              double C1, double C2, double C3) {

        return (A1 * B2 * C3 + B1 * C2 * A3 + C1 * A2 * B3 - C1 * B2 * A3 - B1 * A2 * C3 - A1 * C2 * B3);
    }

    /**
     * Calculates vector wich is normal to v1 and v2
     */
    public void Ort(Vector2 v1, Vector2 v2) {
        double xx = 0;
        double yy = 0;
        double zz = 0;
        double D = det2(v1.y, v1.z, v2.y, v2.z);

        double A;
        double B;

        if (D != 0) {
            xx = 1;
            A = det2(-xx * v1.x, v1.z, -xx * v2.x, v2.z);
            B = det2(v1.y, -xx * v1.x, v2.y, -xx * v2.x);
            yy = A / D;
            zz = B / D;
        } else {
            D = det2(v1.z, v1.x, v2.z, v2.x);
            if (D != 0) {
                yy = 1;
                A = det2(-yy * v1.y, v1.x, -yy * v2.y, v2.x);
                B = det2(v1.z, -yy * v1.y, v2.z, -yy * v2.y);
                zz = A / D;
                xx = B / D;
            } else {
                D = det2(v1.x, v1.y, v2.x, v2.y);
                if (D != 0) {
                    zz = 1;
                    A = det2(-zz * v1.z, v1.y, -zz * v2.z, v2.y);
                    B = det2(v1.x, -zz * v1.z, v2.x, -zz * v2.z);
                    xx = A / D;
                    yy = B / D;
                } else {
                    xx = 0;
                    yy = 0;
                    zz = 0;
                }
            }
        }
        x = xx;
        y = yy;
        z = zz;
    }

    
    /**
     * Calculates the value of an angle with specified sin and cos
     */
    static double acossin(double xsin, double xcos) {
        double ang = Math.acos(xcos);
        if (xsin < 0) {
            ang = 2 * Math.PI - ang;
        }
        return ang;
    }

    /**
     * Ptint to console coordinates of this vector separated by space
     */
    public void print() {
        System.out.println(x + " " + y + " " + z);
    }

}
